function perftra(arg1,arg2,arg3,arg4)
%PERFTRA  Synthesis of a digital perfect tracking feedforward unit (TFI).
%    PERFTRA,g1,g2,g3,g4 - from Transfer Function Interpreter
%    PERFTRA('g1','g2','g3','g4') - from Matlab
%    g1: transfer function of the regulator (input)
%    g2: transfer function of the plant (input)
%    g3: transfer function of the preaction delay (output)
%    g4: transfer function of the feedforward compensator (output)
%    gives as g3 and g4 two feedforward unit that provide perfect tracking
%    in digital feedback systems.

% TFI Environment (A.Civolani & G.Marro) - 1994-96. Matlab5 version, 1997.

global ABIL TRF1 TRF2 TRF3 TRF4 TRF5 TRF5 TRF6 TRF7 HALEG FIGLEG LABLEG SUBPLEG

nfig=gcf; nfigs=int2str(nfig); ffor=0; fpre=0; flgmen=0; flgf=[]; %5.3
FIGLEG=0; HALEG=[]; LABLEG=[]; SUBPLEG=0;
if exist('legen#.mat'), load legen#, else, legen=0; islegen=0; end
[vcol1,vcol2,repco]=coltbl; FIGLO=resfiglo; ABIL=0;
ms1='\n **** error: you cannot change figure while running\n';
titlenable=1;
xx='   **** press any key to continue';
if (nargin~=0)&(nargin~=4)
  disp(' **** error: illegal number of input arguments in perftra'), return
end
if nargin==0
 mess='\n   enter the tf of the regulator (input) : ';
 arg1=input(mess,'s');
 if isempty(arg1), arg1='0'; end, if arg1=='0', return, end
 mess='\n   enter the tf of the plant (input) : ';
 arg2=input(mess,'s');
 if isempty(arg2), arg2='0'; end, if arg2=='0', return, end
 mess='\n   enter the tf of the preaction delay (output) : ';
 arg3=input(mess,'s');
 if isempty(arg3), arg3='0'; end, if arg3=='0', return, end
 mess='\n   enter the tf of the feedforward compensator (output) : ';
 arg4=input(mess,'s');
 if isempty(arg4), arg4='0'; end, if arg4=='0', return, end
end
if ~exist([arg1,'.mat'])
 disp([' **** error: ',arg1,'.mat does not exist']), return
elseif ~exist([arg2,'.mat'])
 disp([' **** error: ',arg2,'.mat does not exist']), return
end
[zr,pr,kr,str1]=exportf(arg1,1); if isempty(kr), return, end
[numr,denr]=dezpk(zr,pr,kr);
if (length(zr)==0)&(length(pr)==0)
 str1='z'; importf(zr,pr,kr,arg1,1)
end
[zp,pp,kp,str2]=exportf(arg2,1); if isempty(kp), return, end
[num,den]=dezpk(zp,pp,kp);
if (str1~='z')|(str2~='z')
 disp(' **** error: perftra applies only to discrete-time systems')
 return
end
g3=arg3; g4=arg4;
figure, tfblff(0,arg1,arg2,arg3,arg4), delete(gcf)

if length(num)>length(den)
 disp(' **** error: m > n in perftra'), return
end

mess1=[arg1,' regulator'];
mess2=[arg2,' plant'];
mess3=[arg3,' delay of the reference'];
mess4=[arg4,' feedforward unit to the input of the plant'];

mess='\n   information on design of perfect tracking compensators ? (1) : ';
ll=input(mess);
if gcf~=nfig, fprintf(ms1), return, end
if isempty(ll), ll=0; end
if ll==1
 figure, tfblff(1,arg1,arg2,arg3,arg4)
 nfig1=gcf; nfigs1=int2str(nfig1);
 clc
 disp(' ')
 disp('   DIGITAL PERFECT TRACKING COMPENSATOR DESIGN')
 disp(' ')
 mess='   The reference block diagram for the design is shown in Fig.';
 mess=[mess,nfigs1];
 disp(mess)
 disp('   click on the figure with mouse to see the details')
 disp('   then click on the command window to recover this text')
 disp(' ')
 disp(['   Transfer function of the regulator : ',arg1]);
 disp(['   Transfer function of the plant : ',arg2]);
 disp(' '), disp(xx), pause
 clc
 disp(' ')
 disp('   1) Derivation of the relative-degree delay D_r :')
 disp(' ')
 disp('   Let us give the plant transfer function the form')
 disp(' ')
 disp('             P(z)    P1(z)')
 disp('      G(z) = ---- = -------- ,   where')
 disp('             Q(z)   z^k*Q(z)')
 disp(' ')
 disp('   1) P(z) is monic - hence Q(z) generally non-monic')
 disp('   2) P1(z) has the same degree as Q(z) - this is obtainable')
 disp('      by multiplying and dividing by z^h for a suitable h')
 disp(' ')
 disp('   The relative-degree delay is defined as D_r := 1/z^k')
 disp(' '), disp(xx), pause
 clc
 disp(' ')
 disp('   2) Derivation of the stable-zero compensator g_s :')
 disp(' ')
 disp('   Define the factorization')
 disp(' ')
 disp('      Q(z)     A(z)     B(z)')
 disp('      ----- = ------ + ------ ,')
 disp('      P1(z)   P1m(z)   P1p(z)')
 disp(' ')
 disp('   where the absolute values of the roots of P1m(z) and P1p(z) are')
 disp('   not greater than 1 and strictly greater than 1 respectively.')
 disp('   A(z) and B(z) can be obtained as A(z):=A1(z), B(z):=z*B1(z),')
 disp('   where A1(z) and B1(z) are provided by the Diophantine equation')
 disp(' ')
 disp('      A1(z)*P1p(z)+ B1(z)*[z*P1m(z)] = Q(z) .')
 disp(' ')
 disp('   The stable-zero compensator is defined as g_s := A(z)/P1m(z)')
 disp(' '), disp(xx), pause
 clc
 disp(' ')
 disp('   3) Derivation of the preaction delay D_p :')
 disp(' ')
 disp('   Let rho be the absolute value of the root of P1p(z) closest')
 disp('   to the unit circle and kp the preaction time interval.')
 disp('   The error reduction obtainable with the unstable-zero compensator')
 disp('   is approximatively given by')
 disp(' ')
 disp('              1')
 disp('      Er = -------- .')
 disp('           rho^(kp)')
 disp(' ')
 disp('   This can be used for selection of kp to obtain a given Er.')
 disp('   The discrete time -kp is the preaction starting time.')
 disp(' ')
 disp('   The preaction delay is defined as D_p := 1/z^(kp)')
 disp(' '), disp(xx), pause
 clc
 disp(' ')
 disp('   4) Derivation of the unstable-zero compensator g_p :')
 disp(' ')
 disp('   Let us consider the Diophantine equation')
 disp(' ')
 disp('      X(z)*z^(kp) + Y(z)*P1p(z) = B1(z) ,')
 disp('  ')
 disp('   where B1(z) has been obtained above (step 2)')
 disp(' ')
 disp('                                                        Y(z)')
 disp('   The unstable-zero compensator is defined as g_p := --------')
 disp('                                                      z^(kp-1)')
 disp(' '), disp(xx), pause
 clc
 disp(' ')
 disp('   4) Derivation of the unstable-zero compensator g_p :')
 disp(' ')
 disp('   The unstable-zero compensator can also be derived as follows:')
 disp('   replace z with 1/z in the transfer function B(z)/P1p(z),')
 disp('   obtained above (step 2) and denote by gr(k) its impulse response')
 disp(' ')
 disp('                                                        Y(z)')
 disp('   The unstable-zero compensator is defined as g_p := --------')
 disp('                                                      z^(kp-1)')
 disp(' ')
 disp('   where the coefficients of Y(z) are the elements of gr(k),')
 disp('   reported as follows: gr(kp)*z^(kp-1)+...+gr(2)*z+gr(1)')
 disp(' '), disp(xx), pause
 clc
 disp(' ')
 disp('   NOTE: Recall the basic factorization presented at step 2 :')
 disp(' ')
 disp('      Q(z)     A(z)     B(z)')
 disp('      ----- = ------ + ------ ,')
 disp('      P1(z)   P1m(z)   P1p(z)')
 disp(' ')
 disp('   If the plant is type p, multiply the function on the left by')
 disp('   z^p/(z-1)^p, then multiply the transfer functions of the obtained')
 disp('   compensators by (z-1)^p/z^p: this contrivance preserves the')
 disp('   asymptotic perfect tracking robustness of the regulation loop')
 disp('   when the signal to be tracked is a step, a ramp, etc., and is')
 disp('   included in the design procedure.')
 disp(' '), disp(xx), pause
 clc
 disp(' ')
 disp('   THE EQUIVALENT OVERALL TRACKING COMPENSATOR')
 figure, tfblff(2,arg1,arg2,arg3,arg4)
 nfig2=gcf; nfigs2=int2str(nfig2);
 disp(' ')
 mess='   The block diagram of the compensator is shown in Fig.';
 mess=[mess,nfigs2];
 disp(mess)
 disp(' ')
 disp(['   Transfer function of the overall delay : ',arg3]);
 disp(['   Transfer function of the overall compensator : ',arg4]);
 disp(' ')
 disp('   These transfer functions are defined in terms of the previous')
 disp('   ones as diplayed in the figure')
 disp(' '), disp(xx), pause
 clc
 disp(' ')
 disp('   DERIVATION OF A SINGLE FEEDFORWARD UNIT')
 figure, tfblff(3,arg1,arg2,arg3,arg4,'FF','bl2')
 nfig3=gcf; nfigs3=int2str(nfig3);
 disp(' ')
 disp('   If the regulator transfer function is invertible, a single')
 mess='   feedforward unit can be used as shown in Fig.';
 mess=[mess,nfigs3];
 disp(mess)
 disp(' ')
 disp('   Its transfer function is defined in terms of the previous')
 disp('   ones as displayed in the figure')
 disp(' '), disp(xx), pause
 delete(nfig1), delete(nfig2), delete(nfig3)
 clc
 flgin=0;
 while flgin==0
  mess='\n   enter 1 to continue with design, 0 to exit: '; xxx=input(mess);
  if isempty(xxx), xxx=4; end
  if gcf~=nfig, fprintf(ms1), return, end
  if xxx==1, flgin=1; elseif xxx==0, ordf(1), return, end
 end
end

% the numerator is made monic
den=den/num(1); num=num/num(1);
m=length(num); n=length(den);

% the relative degree of the plant
k=n-m;

% The transfer function of the plant is: Gp=z^(-k)*num1/den,
% with num1/den strictly proper

num1=[num,zeros(1,k)];

clc, disp(' ')
disp('The transfer functions of the plant and regulator:')
tfm(zp,pp,kp,arg2,1)

tfm(zr,pr,kr,arg1,1)
disp(' ')

fprintf('NOTE: ')
if any(abs(zp)>1)
 fprintf([arg2,' is not invertible: preaction is required'])
 fpre=1;
else
 fprintf([arg2,' is invertible: preaction is not required'])
end
fprintf('\n      ')
if any(abs(zr)>1)
 fprintf([arg1,' is not invertible: the full feedforward unit must be used'])
else
 fprintf([arg1,' is invertible: a simplified feedforward unit can be used'])
 ffor=1;
end
fprintf('\n      ')
fprintf('the delay due to the plant relative degree is D_r=')
fprintf('%.3g samples',k)
ordiff=length(find(abs(pp-1)<10^(-8)));
fprintf('\n      ')
fprintf('compensators are fed with difference of order ')
fprintf('%.3g (type of the plant)\n\n',ordiff)
disp(xx), pause, disp(' ')

% Definition of the preaction time
if fpre
 abszpl=abs(zp);
 domzero=min(abszpl(find(abszpl>1)));
 alpha=10^(-6);
 kkm=ceil(-log10(alpha)/log10(domzero));
 kk=0:1:kkm;
 err=domzero.^(-kk);
 [xx1,yy1]=stairs(kk,err);
 ordf(2), ff=int2str(gcf); figure(gcf) %5.3
 semilogy(xx1,yy1,'Color',vcol1(1,:),'LineStyle','-'), grid
 VV=[0,kkm,alpha,1]; axis(VV)
 mess=['   Fig.',ff,' shows the evaluated tracking error versus'];
 mess=[mess,' the preaction interval'];
 disp(mess)
 flgin=0;
 while flgin==0
  hh=input('\n   enter the preaction interval: kp = ');
  if ~isempty(hh), h=round(abs(hh)); flgin=1; end
 end
 ordf(1)
else
 h=0;
end

clc
disp(' ')
fprintf('Delay due to the plant relative degree: D_r=%.3g samples\n',k)
fprintf('Delay due to preaction: D_p=%.3g samples\n',h)

% correction for difference od order ordiff
P=den; Q=num1;
[zd,pd,kd]=tozpk(P,Q);
for kk=1:ordiff
  zd=[zd;0]; pd=[pd;1];
end

% cancellation of equal poles and zeros
md=length(zd); nd=length(pd);
iz=ones(md,1); ip=ones(nd,1);
for kk=1:md
 for kh=1:nd
  if (abs(zd(kk)-pd(kh))<=10^(-8))&(ip(kh)~=0)
   iz(kk)=0; ip(kh)=0; break
  end
 end
end
zd=zd(find(iz)); pd=pd(find(ip));

if length(zd)~=length(pd)
  disp(' ');
  error(' **** decomposition error in perftra'); return
end

[P,Q]=dezpk(zd,pd,kd);
p=[pd;0];
ii=find(abs(p-1)<10^(-8)); p(ii)=ones(length(ii),1);
Im=find(abs(p)<=1); pm=p(Im); Qm=real(poly(pm));
Ip=find(abs(p)>1); pp=p(Ip); Qp=real(poly(pp));
[Pp,Pm]=diopha(Qm,Qp,P);
Pp=[Pp,0]; Qm=Qm(1:length(Qm)-1);
%ii=find(pm==0); pm=[pm(1:ii-1),pm(ii+1:length(pm))];

% the postaction compensator

nums=Pm; dens=Qm;
disp(' ')
disp('The stable-zero compensator:')
% correction for difference (only in the display)
[zx,px,kx]=tozpk(nums,dens);
for kk=1:ordiff
  zx=[zx;1]; px=[px;0];
end
tfm(zx,px,kx,'g_s',1)

% the preaction compensator

n1=length(Qp); n2=length(Pp);
Pp1=[zeros(1,n1-n2) Pp];
grp=filter([Pp1(n1:-1:1)],Qp(n1:-1:1),[1 zeros(1,h+1)]);
if h>0, nump=grp(h+1:-1:2); else, nump=0; end
denp=[1,zeros(1,h-1)];

disp(' ')
disp('The unstable-zero compensator:')
% correction for difference (only in the display)
nump1=nump; denp1=denp;
for kk=1:ordiff
  nump1=conv(nump1,[1,-1]); denp1=conv(denp1,[1,0]);
end
tfm(nump1,denp1,'g_p',1)
disp(' ')

disp(xx), pause
clc

disp(' ')
disp('The overall delay:')
DD1=[1,zeros(1,h-1)];
numff1=1; denff1=[1,zeros(1,k+h)];
aa=importf(numff1,denff1,arg3,1);
if aa==1, tfg(arg3), end
disp(' ')

numff2=sumpol(conv(nump,[dens,zeros(1,h)]),conv(denp,nums));
denff2=conv(denp,[dens,zeros(1,h)]);

flgg=0;
while flgg==0
 if (numff2(length(numff2))==0)&(denff2(length(denff2))==0)
  numff2=numff2(1:length(numff2)-1); denff2=denff2(1:length(denff2)-1);
 else
  flgg=1;
 end
end

kff2=numff2(1); numff2=numff2/numff2(1);
kff2=kff2/denff2(1); denff2=denff2/denff2(1);

Numc=numff2; Denc=denff2; kc=kff2;
Num1=[1 -1]; Den1=[1 0]; k1=1;

% correction for difference in the overall feedback compensator
for kk=1:ordiff
  [Numc,Denc,kc]=prodg(Numc,Denc,kc,Num1,Den1,k1,0);
%  zd=[zd;1]; pd=[pd,0]
end
aa=stf(Numc,Denc,kc,'z',arg4);
disp(' ')
disp('The feedforward unit to the input of the plant:')
if aa==1, tfg(arg4), end
disp(' ')

disp(xx)
pause

if ffor==1
 flgf=input('\n   do you want to use a single feedforward unit ? (1) : ');
end
if isempty(flgf), flgf=0; end
if flgf==1
% clc
 figure, tfblff(3,arg1,arg2,arg3,arg4,'?','bl2')
 nfig3=gcf; nfigs3=int2str(nfig3);
 flin=0;
 while flin==0
  arg5=input('\n   enter a name for the feedforward transfer function : ','s');
  if length(arg5)~=0, flin=1; end
 end
 tfblff(3,arg1,arg2,arg3,arg4,arg5,'bl2')
 prodgm(arg4,arg1,arg5,1)
 sumgm(arg3,arg5,arg5,0)
 [numff2,denff2]=exportf(arg5);
 flgg=0;
 while flgg==0
  if (numff2(length(numff2))==0)&(denff2(length(denff2))==0)
   numff2=numff2(1:length(numff2)-1); denff2=denff2(1:length(denff2)-1);
  else
   flgg=1;
  end
 end

 aa=importf(numff2,denff2,arg5,1);
 clc
 disp(' ')
 disp('The feedforward unit to the summing junction:')
 if aa==1, tfg(arg5), end
 disp(' ')
 disp(xx)
 pause
% disp(' ')
 delete(gcf)
 mess5=[arg5,' single feedforward unit to the summing junction'];
end

% simulation

flgs=input('\n   do you want to see the system behavior ? (1) : ');
if isempty(flgs), flgs=0; end

if flgs==1

num_numr=conv(num,numr);
den_denr=conv(den,denr);
num_denr=conv(num,denr);
denloop=sumpol(num_numr,den_denr);

% The reference profile to be tracked for benchmarks

flgbench=0;
step=1;
nramp=10;
dist1=30;
dist2=20;

while flgbench==0

 U=[[step/nramp:step/nramp:step],step*ones(1,dist1),zeros(1,dist2)];
 nu=length(U);

% Ud: difference of order ordiff of the reference profile
 Ud=U;
 for kk=1:ordiff
   Ud=[diff([0,Ud])];
 end
% UU: input or difference
 UU=Ud;

% total time for plots
 ntt=[-(h+1):1:k+nu-1]; % a sample added to get a better display

% reference versus total time
 x_rif=[zeros(1,h+1) zeros(1,k) U];
 x_rifd=[zeros(1,h+1) zeros(1,k) UU];

% previewed reference versus total time
 px_rif=[0 U zeros(1,h) zeros(1,k)];
 px_rifd=[0 UU zeros(1,h) zeros(1,k)];

 sc=x_rif;
 smax=max(sc); smin=min(sc);
 delta=0.02*abs(smax-smin);
 if abs(smax-smin)<eps, delta=.001; end
 smin=smin-delta; smax=smax+delta;
 clf, ordf(3)
 if any(FIGLO(8,:))
  erx=0; eval('set(nfig,''Position'',FIGLO(8,1:4))','erx=1;')
 end
 [x1,y1]=stairs(ntt,px_rif); [x2,y2]=stairs(ntt,sc);
 subplot(211)
 ha=plot([0,0],[smin,smax],x1,y1,x2,y2);
 set(ha(1),'Color',vcol1(8,:),'LineStyle','-','Visible','off')
 set(ha(2),'Color',vcol1(3,:),'LineStyle','-')
 set(ha(3),'Color',vcol1(1,:),'LineStyle','-')
 VV=axis; VV(1:2)=[min(ntt),max(ntt)]; axis(VV), grid, xlabel(mess)
 aat=setstr(32*ones(1,120)); aam=sprintf('k');
 mess=[aat(1:120-length(aam)),aam]; xlabel(mess)
 if titlenable==1
  title('previewed reference (c) and reference (g)')
 end
 sc=px_rifd;
 smax=max(sc); smin=min(sc);
 delta=0.02*abs(smax-smin);
 if abs(smax-smin)<eps, delta=.001; end
 smin=smin-delta; smax=smax+delta;
 [x1,y1]=stairs(ntt,sc);
 subplot(212)
 ha=plot([0,0],[smin,smax],x1,y1);
 set(ha(1),'Color',vcol1(8,:),'LineStyle','none','Marker,'.')
 set(ha(2),'Color',vcol1(3,:),'LineStyle','-')
% VV=axis; VV(1:2)=[min(ntt),max(ntt)];
 axis(VV), grid, xlabel(mess)
 if titlenable==1
  title('difference used to derive the input corrrection')
 end
 nfig=gcf; nfigs=int2str(nfig); figure(nfig)
 xyx=input(['\n   **** figure ',nfigs,' **** '],'s');
 FFLOC=get(nfig,'Position');
 clc
 fff=input('\n   do you want to change the parameters of the profile ? (1) : ');
 if isempty(fff), fff=0; end
 if fff==1
  mess='\n   enter the constant value after the ramp (default=1) : ';
  step1=input(mess);
  if length(step1)==0, step=1; else, step=step1; end
  mess='\n   enter the number of samples in the ramp (default=10) : ';
  nramp1=input(mess);
  if length(nramp1)==0, nramp=10; else, nramp=abs(nramp1); end
  mess='\n   enter the number of samples between ramp and step (default=30) : ';
  dist11=input(mess);
  if length(dist11)==0, dist1=30; else, dist1=abs(dist11); end
  mess='\n   enter the number of samples after the step (default=20) : ';
  dist21=input(mess);
  if length(dist21)==0, dist2=20; else, dist2=abs(dist21); end
 else
  flgbench=1;
 end
end

subplot(2,1,1), title(''), subplot(2,1,2), title('')

FFLOC=get(nfig,'Position');
% postaction profile
ttt=[zeros(1,length(Qm)-length(Pm)),Pm];
gf=filter(ttt,Qm,[1,zeros(1,nu-1+k)]);

% postaction compensation
ttt=[zeros(1,length(Qm)-length(Pm)),Pm];
u_cpost=filter(ttt,Qm,UU);
u_cpost=[zeros(1,h+1) u_cpost zeros(1,k)];

% preaction profile
Pp1=[zeros(1,n1-n2) Pp];
gr=filter([Pp1(n1:-1:1)],Qp(n1:-1:1),[1 zeros(1,h+k+nu)]);

% preaction compensation
u_cpre=zeros(1,h+k+nu);
for i=1:nu
  u_cpre=u_cpre+[zeros(1,i-1) UU(i)*gr(h+1:-1:2) zeros(1,k) zeros(1,nu-i+1)];
end
u_cpre=[0 u_cpre];
gr=[gr(h+1:-1:1) zeros(1,nu+k-1)];

% total profile
gr=[0 gr];
gf=[zeros(1,h+1) gf];
gtot=gr+gf;

% output without any correction
ttt=[zeros(1,length(denloop)-length(num_numr)),num_numr];
y_rif=filter(ttt,denloop,x_rif);

% input without any correction
numr_den=conv(numr,den);
ttt=[zeros(1,length(denloop)-length(numr_den)),numr_den];
u_rif=filter(ttt,denloop,x_rif);

% error without any correction
e_rif=x_rif-y_rif;

% total input correction
u_ctot=u_cpre+u_cpost;

% output due to the total input correction
ttt=[zeros(1,length(denloop)-length(num_denr)),num_denr];
y_ctot=filter(ttt,denloop,u_ctot);

% output due to reference and total correction
y_rif_ctot=y_rif+y_ctot;

% error due to reference and total correction
e_rif_ctot=x_rif-y_rif_ctot;

% influence of total correction on input (closed-loop)
ttt=[zeros(1,length(denloop)-length(den_denr)),den_denr];
u_cclo=filter(ttt,denloop,u_ctot);

% influence of reference and total correction on input (closed-loop)
u_rif_ctot=u_rif+u_cclo;

% reference names for plots
nome=['x_rif      '];
nome=[nome;'gr         ';'gf         ';'gtot       '];
nome=[nome;'u_cpre     ';'u_cpost    ';'u_ctot     '];
nome=[nome;'u_rif      ';'u_rif_ctot '];
nome=[nome;'y_rif      ';'y_rif_ctot '];
nome=[nome;'e_rif      ';'e_rif_ctot '];

options=[];
options=[options;'reference input (delayed by the plant relative degree)'];
options=[options;'preaction for an impulse of the selected difference   '];
options=[options;'postaction for an impulse of the selected difference  '];
options=[options;'correction for an impulse of the selected difference  '];
options=[options;'preaction for the whole reference                     '];
options=[options;'postaction for the whole reference                    '];
options=[options;'correction for the whole reference                    '];
options=[options;'input of the plant without correction                 '];
options=[options;'input of the plant with correction                    '];
options=[options;'output of the plant without correction                '];
options=[options;'output of the plant with correction                   '];
options=[options;'tracking error without correction                     '];
options=[options;'tracking error with correction                        '];
options=[options;'define a new graphic window                           '];

vcol=[2;3;3;3;3;3;3;6;6;1;1;5;5];

% managing the available plots

esci=0; flg=0;

while esci==0

clc
disp(' ')
disp('   THE AVAILABLE PLOTS :')
disp(' ')
for kk=1:14
 tit1=options(kk,:); tit1=deblank(tit1);
 if kk<10, sz='    '; else, sz='   '; end
 disp([sz,int2str(kk),' - ',tit1])
end

in=input('\n   enter your choice (press return to exit) : ');
if isempty(in)
 cc=get(0,'Children');
 if any(cc==gcf)
  FIGLO(8,1:4)=FFLOC;
  save figlo# FIGLO -v4
 else
  disp(' '), disp('   THE FIGURE HAS BEEN DELETED!')
 end
 eval('delete(men_gr)','err=1;'), eval('delete(men_zoom)','err=1;')
 ordf(1), ABIL=0; return
else
 in=round(in);
 if (in>=1)&(in<=13)
  nome1=nome(in,:); nome1=nome1(find(nome1~=' '));
  tit1=options(in,:); tit1=deblank(tit1); col=vcol(in);
  eval(['s=',nome1,';']); smax=max(s); smin=min(s);
  delta=0.02*abs(smax-smin);
  if abs(smax-smin)<eps, delta=.001; end
  smin=smin-delta; smax=smax+delta;
  if (flg==0)&(flgmen==0)
   men_opt=uimenu(gcf,'Label','Recall');
   mess='digital feedforward compensation for perfect tracking';
   uimenu(men_opt,'Label',mess);
   TRF1=[arg1,'g'];
   uimenu(men_opt,'Label',mess1,'Separator','on',...
   'Callback','subplot(2,1,1),tfmaux(1,1,0)');
   TRF2=[arg2,'b'];
   uimenu(men_opt,'Label',mess2,...
   'Callback','subplot(2,1,1),tfmaux(2,1,0)');
   TRF3=[arg3,'c'];
   uimenu(men_opt,'Label',mess3,...
   'Callback','subplot(2,1,1),tfmaux(3,1,0)');
   TRF4=[arg4,'c'];
   uimenu(men_opt,'Label',mess4,...
   'Callback','subplot(2,1,1),tfmaux(4,1,0)');
   men_gr=uimenu(gcf,'Label','Reticolo on/off','Callback','gridmng(6)');
   men_zoom=uimenu(gcf,'Label','Zoom on/off','Callback','zoom');
   if flgf==1
    TRF5=[arg5,'c'];
    uimenu(men_opt,'Label',mess5,...
    'Callback','subplot(2,1,1),tfmaux(5,1,0)');
   end
   flgmen=1;
  end
  if flg==0
   ordf(3), set(gcf,'Position',FFLOC)
   subplot(2,1,1), cla, set(gca,'Box','on')
   subplot(2,1,2), cla, set(gca,'Box','on'), title('')
   [x1,y1]=stairs(ntt,s);
   subplot(2,1,1), reset(gca)
   plot([0,0],[smin,smax],'Color',vcol1(8,:),'LineStyle','-','Visible','off')
   hold on, plot(x1,y1,'Color',vcol1(col,:),'LineStyle','-')
   VV=axis; VV(1:2)=[min(ntt),max(ntt)]; axis(VV), grid
   if titlenable==1, title(tit1), end
   aat=setstr(32*ones(1,120)); aam=sprintf('k');
   mess=[aat(1:120-length(aam)),aam]; xlabel(mess)
   figure(gcf)
   nfigs=int2str(gcf); ABIL=2.2;
   xyx=input(['\n   **** figure ',nfigs,' **** '],'s');
   flg=1;
  else
   hold off
   [x1,y1]=stairs(ntt,s);
   subplot(2,1,2), reset(gca)
   plot([0,0],[smin,smax],'Color',vcol1(8,:),'LineStyle','-','Visible','off')
   hold on, plot(x1,y1,'Color',vcol1(col,:),'LineStyle','-')
   VV=axis; VV(1:2)=[min(ntt),max(ntt)]; axis(VV), grid
   if titlenable==1, title(tit1), end
   aat=setstr(32*ones(1,120)); aam=sprintf('k');
   mess=[aat(1:120-length(aam)),aam]; xlabel(mess)
   figure(gcf)
   nfigs=int2str(gcf);
   xyx=input(['\n   **** figure ',nfigs,' **** '],'s');
   FFLOC=get(gcf,'Position');
   flg=0;
  end
 elseif in==14
  eval('delete(men_gr)','err=1;'), eval('delete(men_zoom)','err=1;')
  ordf(1), figure, ordf(1), flg=0; flgmen=0;
 end
 end % while
end % if length(in)==0
end % if flgs==1
% --- last line of perftra ---
