function regdph(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8)
%REGDPH Regulator synthesis by closed-loop pole allocation (TFI).
%   REGDPH,g1,g2,g3,g4 - from Transfer Function Interpreter
%   REGDPH('g1','g2','g3','g4') - from Matlab
%   with g1, g2 and g3 inputs, g4 output
%   g1: transfer function of the plant (with n poles)
%   g2: transfer function of the fixed part of the regulator
%	(with mr zeros and nr poles)
%   g3: transfer function with the nt desired closed-loop poles
%	(nt >= 2n+mr+nr-1); zeros and gain are not considered
%   g4: the derived transfer function, such that g4*g1/(1+g4*g1)
%	has the same poles as g3.
%   NOTE: REGDPH,g1,g2 from TFI -- or REGDPH('g1','g2') from Matlab --
%	  displays the minimum number of poles to assign in g3 in order
%	  the regulator transfer function g4 to be (strictly) proper.

% TFI Environment (A.Civolani & G.Marro) - 1994-96. Matlab5 version, 1997.

err='err'; z5=[]; p5=[];
nargs=nargin;
if (nargs==1)|(nargs==3)|(nargs>4)
 disp(' **** error: illegal number of input arguments in regdph'), return
end
xxx=nargs;
if xxx==0
 mess='\n   enter the tf of the plant (input) : ';
 arg1=input(mess,'s'); if isempty(arg1), arg1='0'; end
 if arg1=='0', return, end
 mess='\n   enter the tf of the fixed part of the regulator (input) : ';
 arg2=input(mess,'s'); if isempty(arg2), arg2='0'; end
 if arg2=='0', return, end
 mess='\n   enter the tf with the desired closed-loop poles (input) : ';
 arg3=input(mess,'s'); if isempty(arg3), arg3='0'; end
 if arg3=='0', return, end
 mess='\n   enter the tf of the regulator (output) : ';
 arg4=input(mess,'s'); if isempty(arg4), arg4='0'; end
 if arg4=='0', return, end
 nargs=4;
end
if ~exist([arg1,'.mat'])
 disp([' **** error: ',arg1,'.mat does not exist']), return
end
if nargs>2
 if ~exist([arg3,'.mat'])
  disp([' **** error: ',arg3,'.mat does not exist']), return
 end
end
[num1,den1,str1]=exportf(arg1,1);
[z1,p1,k1]=exportf(arg1); if isempty(k1), return, end
st1=arg2(1);
rop=(abs(st1)>=48)&(abs(st1)<=57);
if rop
 st=eval(arg2,'err');
 if isstr(st)
  disp(' **** input error in regdph'), return
 end
else
 if ~exist([arg2,'.mat'])
  disp([' **** error: ',arg2,'.mat does not exist']), return
 end
 st=arg2;
end
if isstr(st)
 [num2,den2,str2]=exportf(arg2,1);
 [z2,p2,k2]=exportf(arg2); if isempty(k2), return, end
else
 num2=abs(st);den2=1; str2=str1; z2=[]; p2=[]; k2=abs(st);
end
if nargs>2
 [num3,den3,str3]=exportf(arg3,1); if isempty(den3), return, end
 if str2~=str1 | str3~=str1
  disp(' **** error: mixed continuous and discrete tf''s'), return
 end
 figure, tfblocks(3,arg4,arg1,'','','','','','','',''), delete(gcf), g4=arg4;
else
 if str2~=str1
  disp(' **** error: mixed continuous and discrete tf''s'), return
 end
end
%
%   selection of common factors
%
tol=10^(-8);
md=length(z2); nd=length(p1);
iz=ones(md,1); ip=ones(nd,1);
for kk=1:md
 for kh=1:nd
  if (abs(z2(kk)-p1(kh))<=tol)&(ip(kh)~=0)
   iz(kk)=0; ip(kh)=0; z5=[z5;z2(kk)]; break
  end
 end
end
z2=z2(find(iz)); p1=p1(find(ip));
md=length(p2); nd=length(z1);
iz=ones(md,1); ip=ones(nd,1);
for kk=1:md
 for kh=1:nd
  if (abs(p2(kk)-z1(kh))<=tol)&(ip(kh)~=0)
   iz(kk)=0; ip(kh)=0; p5=[p5;p2(kk)]; break
  end
 end
end
p2=p2(find(iz)); z1=z1(find(ip));
if (length(z5)>0) | (length(p5)>0)
 [num1,den1]=dezpk(z1,p1,k1);
 [num2,den2]=dezpk(z2,p2,k2);
 tfm(z5,p5,1,'   regulator factors cancellable with plant')
 disp(' '), disp('   **** press any key to continue'), pause
end
%
if length(num1)>length(den1)
 disp(' '), disp('   **** warning: m > n')
end
np=length(p1); nz2=length(z2); np2=length(p2);
nz5=length(z5); np5=length(p5); ll3=2*np+nz2+np2-1+nz5-np5;
if nargs==2
 mess='   the minimum number of poles to assign ';
 mess=[mess,'for a causal regulator is ',int2str(ll3)];
 disp(' '), disp(mess)
 return
end
np3=length(den3)-1;
if np3 < ll3
 disp(' '), disp('   **** warning: non-proper regulator')
elseif np3 > ll3
 disp(' ')
 mess='   strictly proper regulator: the assigned poles may be reduced by ';
 disp([mess,int2str(np3-ll3)])
end
%
%   Solution of the Diophantine equation
%
aa=conv(den1,den2); bb=conv(num1,num2); cc=den3;
[xx,yy]=diopha(aa,bb,cc);
if ~any(xx)
 disp('   **** solution impossible: the computed denominator is zero !')
 return
end
num4=conv(yy,num2); den4=conv(xx,den2);
[z4,p4,k4]=tozpk(num4,den4);
%
%   recovering the selected common factors
%
z4=[z4;z5]; p4=[p4;p5];
%
if str1=='s'
 aa=importf(z4,p4,k4,g4);
else
 aa=importf(z4,p4,k4,g4,1);
end
if aa==1, tfg(g4), end
% --- last line of regdph ---
